const _ = require("lodash");
const Dialog = require("cattura-ui-library/src/lib/ui/dialog");
const ko = require("knockout");
const moment = require("moment");

const template = require("./content.pug");

const MediaPackage = require("cattura-ui-library/src/lib/mediaPackage");
const selectable = require("cattura-ui-library/src/lib/ui/selectable");
const {apiv1} = require("./api");

const {makeMoment} = require("./time");

let url = ko.observable();
let step = ko.observable("pull");
let error = ko.observable();

let isLoading = ko.observable(false);
let isImporting = ko.observable(false);

let mediaPackages = selectable.selectableObservableArray();
let totalMediaPackages = ko.observable();
let currentPage = -1;
let perPage = 10;

function extendMediaPackage (mediaPackage)
{
    mediaPackage.root = `${url()}/${mediaPackage.root}`;
    mediaPackage.creationDate = makeMoment(mediaPackage.creationDate);
    
    if (!(_(mediaPackage.root).startsWith("http://") || _(mediaPackage.root).startsWith("https://")))
    {
        mediaPackage.root = "http://" + mediaPackage.root;
    }
    
    if (mediaPackage.duration)
    {
        mediaPackage.duration = moment.duration(mediaPackage.duration.timestamp, mediaPackage.duration.timeunit);
    }    
    
    return mediaPackage;
}

/*$('.import-recs-filter-bar-filter-icon').on('click', function () {
    if ($(this).hasClass('icon-filter')) {
        $(this).removeClass('icon-filter').addClass('icon-filter-sel');
        $('.import-recs-filter-subbar').removeClass('is-hidden');
    }
    else {
        $(this).removeClass('icon-filter-sel').addClass('icon-filter');
        $('.import-recs-filter-subbar').addClass('is-hidden');
    }
});*/

function clear ()
{
    mediaPackages([]);
    totalMediaPackages(0);
    currentPage = -1;
    url("");
    step("pull");
    error(null);
    isLoading(false);
    isImporting(false);
}

function makeDialog (plugin)
{
    function loadPage (page)
    {
        isLoading(true);
        return plugin.services.fetchMediaPackagesFromUnit({
            sourceUnitURL: url(),
            page: page
        }).then(function (result) {
            var newMediaPackages = _(result.mediaPackages)
               .map(MediaPackage)
               .map(extendMediaPackage)
               .map(mediaPackage => {
                   if (mediaPackage.hasMedia)
                   {
                       selectable.makeSelectable(mediaPackage);
                   }
                   
                   return mediaPackage;
               })
               .value();

            totalMediaPackages(result.total);
            currentPage = result.page;
            mediaPackages(_(mediaPackages()).concat(newMediaPackages).value());
        }).finally(() => isLoading(false));
    }

    function loadMore ()
    {
        return loadPage(currentPage + 1);
    }
    
    function importSelectedMediaPackages ()
    {
        isImporting(true);
        return plugin.services.pullMediaPackages({
            sourceUnitURL: url(),
            mediaPackageIds: _(mediaPackages.selected()).map(mediaPackage => mediaPackage.id).value()
        }).then(() => isImporting(false));
    }

    function pullRemoteMediaPackages ()
    {
        loadMore().then(() => {
            step("import");
        }).catch((err) => {
            console.error(err);
            error(err);
        });
    }

    return Dialog({
        open ()
        {
            clear();
            this.show();
            this.getElement().find(".url-input").focus();
        },

        url: url,
        step: step,
        error: error,
        isLoading: isLoading,
        isImporting: isImporting,

        mediaPackages: mediaPackages,
        totalMediaPackages: totalMediaPackages,
        pullRemoteMediaPackages: pullRemoteMediaPackages,

        submit ()
        {
            return importSelectedMediaPackages().then(() => this.hide());
        },

        loadMore: loadMore
    }, {
        template: template
    });
}

module.exports = makeDialog;
